package partisan.weforge.xyz.pulse

import android.graphics.Rect
import android.os.Build
import android.os.Bundle
import android.util.DisplayMetrics
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.ItemTouchHelper
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.core.content.getSystemService
import partisan.weforge.xyz.pulse.databinding.FragmentPopupSettingsBinding

class PopupSettingsFragment : Fragment() {

    private var _binding: FragmentPopupSettingsBinding? = null
    private val binding get() = _binding!!

    private lateinit var prefs: Preferences
    private lateinit var window: PopupWindow

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentPopupSettingsBinding.inflate(inflater, container, false)
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        val services = listOf(
            ServiceEntry(
                "vnd.android.cursor.item/vnd.org.thoughtcrime.securesms.call",
                R.string.destination_signal,
                requireContext().isServiceEnabled("vnd.android.cursor.item/vnd.org.thoughtcrime.securesms.call")
            ),
            ServiceEntry(
                "vnd.android.cursor.item/vnd.org.telegram.messenger.android.call",
                R.string.destination_telegram,
                requireContext().isServiceEnabled("vnd.android.cursor.item/vnd.org.telegram.messenger.android.call")
            ),
            ServiceEntry(
                "vnd.android.cursor.item/vnd.ch.threema.app.call",
                R.string.destination_threema,
                requireContext().isServiceEnabled("vnd.android.cursor.item/vnd.ch.threema.app.call")
            ),
            ServiceEntry(
                "vnd.android.cursor.item/vnd.com.whatsapp.voip.call",
                R.string.destination_whatsapp,
                requireContext().isServiceEnabled("vnd.android.cursor.item/vnd.com.whatsapp.voip.call")
            ),
        )

        val adapter = ServiceAdapter(
            context = requireContext(),
            services = services.toMutableList(),
            onReordered = { updatedList ->
                updatedList.forEachIndexed { index, entry ->
                    requireContext().setServicePriority(entry.mimetype, index)
                }
            }
        )

        binding.serviceRecycler.adapter = adapter
        binding.serviceRecycler.layoutManager = LinearLayoutManager(requireContext())

        val touchHelper = ItemTouchHelper(adapter.dragHelper)
        touchHelper.attachToRecyclerView(binding.serviceRecycler)

        adapter.setDragStartListener { viewHolder ->
            touchHelper.startDrag(viewHolder)
        }
    }

    private fun getScreenHeightPx(): Int {
        val wm = requireContext().getSystemService<android.view.WindowManager>()!!
        return if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
            val bounds: Rect = wm.currentWindowMetrics.bounds
            bounds.height()
        } else {
            val metrics = DisplayMetrics()
            @Suppress("DEPRECATION")
            requireActivity().windowManager.defaultDisplay.getMetrics(metrics)
            metrics.heightPixels
        }
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }
}
