package partisan.weforge.xyz.pulse

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.ItemTouchHelper
import androidx.recyclerview.widget.LinearLayoutManager
import partisan.weforge.xyz.pulse.databinding.FragmentServiceSettingsBinding

class ServiceSettingsFragment : Fragment() {

    private var _binding: FragmentServiceSettingsBinding? = null
    private val binding get() = _binding!!

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentServiceSettingsBinding.inflate(inflater, container, false)
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        val services = listOf(
            ServiceEntry(
                "vnd.android.cursor.item/vnd.org.thoughtcrime.securesms.call",
                R.string.destination_signal,
                requireContext().isServiceEnabled("vnd.android.cursor.item/vnd.org.thoughtcrime.securesms.call")
            ),
            ServiceEntry(
                "vnd.android.cursor.item/vnd.org.telegram.messenger.android.call",
                R.string.destination_telegram,
                requireContext().isServiceEnabled("vnd.android.cursor.item/vnd.org.telegram.messenger.android.call")
            ),
            ServiceEntry(
                "vnd.android.cursor.item/vnd.ch.threema.app.call",
                R.string.destination_threema,
                requireContext().isServiceEnabled("vnd.android.cursor.item/vnd.ch.threema.app.call")
            ),
            ServiceEntry(
                "vnd.android.cursor.item/vnd.com.whatsapp.voip.call",
                R.string.destination_whatsapp,
                requireContext().isServiceEnabled("vnd.android.cursor.item/vnd.com.whatsapp.voip.call")
            ),
        )

        val adapter = ServiceAdapter(
            context = requireContext(),
            services = services.toMutableList(),
            onReordered = { updatedList ->
                updatedList.forEachIndexed { index, entry ->
                    requireContext().setServicePriority(entry.mimetype, index)
                }
            }
        )

        binding.serviceRecycler.adapter = adapter
        binding.serviceRecycler.layoutManager = LinearLayoutManager(requireContext())

        val touchHelper = ItemTouchHelper(adapter.dragHelper)
        touchHelper.attachToRecyclerView(binding.serviceRecycler)

        adapter.setDragStartListener { viewHolder ->
            touchHelper.startDrag(viewHolder)
        }
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }
}
